@@ -25,10 +25,10 @@ struct reciprocal_value {
 	u8 sh1, sh2;
 };
 
-struct reciprocal_value reciprocal_value(u32 d);
+struct reciprocal_value rpl_reciprocal_value(u32 d);
 
 #define reciprocal_divide rpl_reciprocal_divide
-static inline u32 reciprocal_divide(u32 a, struct reciprocal_value R)
+static inline u32 rpl_reciprocal_divide(u32 a, struct reciprocal_value R)
 {
 	u32 t = (u32)(((u64)a * R.m) >> 32);
 	return (t + ((a - t) >> R.sh1)) >> R.sh2;
