@@ -11,23 +11,28 @@
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,3,0)
 #define ipv6_skip_exthdr rpl_ipv6_skip_exthdr
-extern int ipv6_skip_exthdr(const struct sk_buff *skb, int start,
-			    u8 *nexthdrp, __be16 *frag_offp);
+extern int rpl_ipv6_skip_exthdr(const struct sk_buff *skb, int start,
+			        u8 *nexthdrp, __be16 *frag_offp);
 #endif
 
+#ifndef HAVE_IP6_FH_F_SKIP_RH
+
 enum {
-	OVS_IP6T_FH_F_FRAG	= (1 << 0),
-	OVS_IP6T_FH_F_AUTH	= (1 << 1),
-	OVS_IP6T_FH_F_SKIP_RH	= (1 << 2),
+	IP6_FH_F_FRAG           = (1 << 0),
+	IP6_FH_F_AUTH           = (1 << 1),
+	IP6_FH_F_SKIP_RH        = (1 << 2),
 };
 
 /* This function is upstream, but not the version which skips routing
- * headers with 0 segments_left. We plan to propose the extended version. */
+ * headers with 0 segments_left. We fixed it when we introduced
+ * IP6_FH_F_SKIP_RH.
+ */
 #define ipv6_find_hdr rpl_ipv6_find_hdr
-extern int ipv6_find_hdr(const struct sk_buff *skb, unsigned int *offset,
-			 int target, unsigned short *fragoff, int *fragflg);
+extern int rpl_ipv6_find_hdr(const struct sk_buff *skb, unsigned int *offset,
+			     int target, unsigned short *fragoff, int *fragflg);
+#endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3,4,0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,6,0)
 static inline u32 ipv6_addr_hash(const struct in6_addr *a)
 {
 #if defined(CONFIG_HAVE_EFFICIENT_UNALIGNED_ACCESS) && BITS_PER_LONG == 64
@@ -42,4 +47,11 @@ static inline u32 ipv6_addr_hash(const struct in6_addr *a)
 }
 #endif
 
+#if defined(OVS_FRAGMENT_BACKPORT) && !defined(HAVE___IPV6_ADDR_JHASH)
+static inline u32 __ipv6_addr_jhash(const struct in6_addr *a, const u32 unused)
+{
+       return ipv6_addr_jhash(a);
+}
+#endif
+
 #endif
