@@ -6,6 +6,7 @@
 #include <linux/mm.h>
 #include <linux/net.h>
 #include <net/checksum.h>
+#include <net/ip.h>
 #include <linux/string.h>
 #include <linux/types.h>
 #include <linux/percpu.h>
@@ -18,9 +19,9 @@
 #include <asm/uaccess.h>
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
-void inet_proto_csum_replace16(__sum16 *sum, struct sk_buff *skb,
-			       const __be32 *from, const __be32 *to,
-			       int pseudohdr)
+void rpl_inet_proto_csum_replace16(__sum16 *sum, struct sk_buff *skb,
+			           const __be32 *from, const __be32 *to,
+			           int pseudohdr)
 {
 	__be32 diff[] = {
 		~from[0], ~from[1], ~from[2], ~from[3],
@@ -36,9 +37,12 @@ void inet_proto_csum_replace16(__sum16 *sum, struct sk_buff *skb,
 		*sum = ~csum_fold(csum_partial(diff, sizeof(diff),
 					csum_unfold(*sum)));
 }
+EXPORT_SYMBOL_GPL(rpl_inet_proto_csum_replace16);
 #endif
 
-bool __net_get_random_once(void *buf, int nbytes, bool *done,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,13,0)
+
+bool rpl___net_get_random_once(void *buf, int nbytes, bool *done,
 			   atomic_t *done_key)
 {
 	static DEFINE_SPINLOCK(lock);
@@ -58,3 +62,6 @@ bool __net_get_random_once(void *buf, int nbytes, bool *done,
 
 	return true;
 }
+EXPORT_SYMBOL_GPL(rpl___net_get_random_once);
+
+#endif
