@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 Nicira Networks.
+ * Copyright (c) 2010, 2011, 2012, 2013, 2014 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,21 +18,26 @@
 #define MULTIPATH_H 1
 
 #include <stdint.h>
+#include "compiler.h"
+#include "ofp-errors.h"
 
 struct ds;
 struct flow;
+struct flow_wildcards;
 struct nx_action_multipath;
-struct nx_action_reg_move;
+struct ofpact_multipath;
+struct ofpbuf;
 
-/* NXAST_MULTIPATH helper functions.
- *
- * See include/openflow/nicira-ext.h for NXAST_MULTIPATH specification.
- */
+/* NXAST_MULTIPATH helper functions. */
 
-int multipath_check(const struct nx_action_multipath *, const struct flow *);
-void multipath_execute(const struct nx_action_multipath *, struct flow *);
+enum ofperr multipath_check(const struct ofpact_multipath *,
+                            const struct flow *);
 
-void multipath_parse(struct nx_action_multipath *, const char *);
-void multipath_format(const struct nx_action_multipath *, struct ds *);
+void multipath_execute(const struct ofpact_multipath *, struct flow *,
+                       struct flow_wildcards *);
+
+char *multipath_parse(struct ofpact_multipath *, const char *)
+    OVS_WARN_UNUSED_RESULT;
+void multipath_format(const struct ofpact_multipath *, struct ds *);
 
 #endif /* multipath.h */
