@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2011 Nicira Networks
+/* Copyright (c) 2009, 2011, 2013, 2015 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -126,6 +126,15 @@ ovsdb_parser_has_error(const struct ovsdb_parser *parser)
     return parser->error != NULL;
 }
 
+struct ovsdb_error *
+ovsdb_parser_destroy(struct ovsdb_parser *parser)
+{
+    free(parser->name);
+    sset_destroy(&parser->used);
+
+    return parser->error;
+}
+
 struct ovsdb_error *
 ovsdb_parser_finish(struct ovsdb_parser *parser)
 {
@@ -142,7 +151,7 @@ ovsdb_parser_finish(struct ovsdb_parser *parser)
                     if (n_unused > 1) {
                         ovsdb_parser_raise_error(
                             parser,
-                            "Member '%s' and %zu other member%s "
+                            "Member '%s' and %"PRIuSIZE" other member%s "
                             "are present but not allowed here.",
                             node->name, n_unused - 1, n_unused > 2 ? "s" : "");
                     } else {
@@ -157,8 +166,5 @@ ovsdb_parser_finish(struct ovsdb_parser *parser)
         }
     }
 
-    free(parser->name);
-    sset_destroy(&parser->used);
-
-    return parser->error;
+    return ovsdb_parser_destroy(parser);
 }
