@@ -29,7 +29,7 @@ struct pstream;
 struct sockaddr_storage;
 
 int new_fd_stream(const char *name, int fd, int connect_status,
-                  struct stream **streamp);
+                  int fd_type, struct stream **streamp);
 int new_fd_pstream(const char *name, int fd,
                    int (*accept_cb)(int fd, const struct sockaddr_storage *ss,
                                     size_t ss_len, struct stream **),
