@@ -19,6 +19,8 @@
 
 struct dpif;
 struct dpif_backer;
+struct dpif_upcall;
+struct ofpbuf;
 struct seq;
 struct simap;
 
@@ -26,7 +28,9 @@ struct simap;
  * them.  Additionally, it's responsible for maintaining the datapath flow
  * table. */
 
+void udpif_init(void);
 struct udpif *udpif_create(struct dpif_backer *, struct dpif *);
+void udpif_run(struct udpif *udpif);
 void udpif_set_threads(struct udpif *, size_t n_handlers,
                        size_t n_revalidators);
 void udpif_synchronize(struct udpif *);
@@ -34,6 +38,6 @@ void udpif_destroy(struct udpif *);
 void udpif_revalidate(struct udpif *);
 void udpif_get_memory_usage(struct udpif *, struct simap *usage);
 struct seq *udpif_dump_seq(struct udpif *);
-void udpif_flush(void);
+void udpif_flush(struct udpif *);
 
 #endif /* ofproto-dpif-upcall.h */
