@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2009, 2010, 2013 Nicira, Inc.
+ * Copyright (c) 2008, 2009, 2010, 2013, 2014 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,15 +18,15 @@
  * hmap.h. */
 
 #include <config.h>
+#undef NDEBUG
 #include "hmap.h"
+#include <assert.h>
 #include <string.h>
 #include "hash.h"
+#include "ovstest.h"
 #include "random.h"
 #include "util.h"
 
-#undef NDEBUG
-#include <assert.h>
-
 /* Sample hmap element. */
 struct element {
     int value;
@@ -285,13 +285,13 @@ run_test(void (*function)(hash_func *))
     }
 }
 
-int
-main(void)
+static void
+test_hmap_main(int argc OVS_UNUSED, char *argv[] OVS_UNUSED)
 {
     run_test(test_hmap_insert_delete);
     run_test(test_hmap_for_each_safe);
     run_test(test_hmap_reserve_shrink);
     printf("\n");
-    return 0;
 }
 
+OVSTEST_REGISTER("test-hmap", test_hmap_main);
